@@ -294,13 +294,14 @@ class ReverseWrapper(object):
         if len(self.pyret_parse_items) == 1:
             ## if retval is one item only, pack it in a tuple so we
             ## can use PyArg_ParseTuple as usual..
-            self.write_code('py_retval = Py_BuildValue("N", py_retval);')
+            self.write_code('py_retval = Py_BuildValue("(N)", py_retval);')
         if len(self.pyret_parse_items) > 0:
             ## Parse return values using PyArg_ParseTuple
+            params = ["py_retval",
+                      '"%s"' % "".join([format for format, param in self.pyret_parse_items])]
+            params.extend([param for format, param in self.pyret_parse_items if param])
             self.write_code(code=None, failure_expression=(
-                '!PyArg_ParseTuple(py_retval, "%s", %s)' % (
-                "".join([format for format, param in self.pyret_parse_items]),
-                ", ".join([param for format, param in self.pyret_parse_items]))))
+                '!PyArg_ParseTuple(%s)' % (', '.join(params),)))
 
         if DEBUG_MODE:
             self.declarations.writeln("/* end declarations */")
@@ -394,10 +395,12 @@ del ctype
 class StringReturn(ReturnType):
 
     def get_c_type(self):
-        return "char *"
+        return self.props.get('c_type', 'char *').replace('const-', 'const ')
+    #return "char *"
 
     def write_decl(self):
-        self.wrapper.add_declaration("char *retval;")
+        self.wrapper.add_declaration("%s retval;" % self.get_c_type())
+        #self.wrapper.add_declaration("char *retval;")
 
     def write_error_return(self):
         self.wrapper.write_code("return NULL;")
@@ -406,8 +409,8 @@ class StringReturn(ReturnType):
         self.wrapper.add_pyret_parse_item("s", "&retval", prepend=True)
         self.wrapper.write_code("retval = g_strdup(retval);", code_sink=self.wrapper.post_return_code)
 
-for ctype in ('char*', 'gchar*'):
-    argtypes.matcher.register_reverse(ctype, StringReturn)
+for ctype in ('char*', 'gchar*', 'const-gchar*'):
+    argtypes.matcher.register_reverse_ret(ctype, StringReturn)
 del ctype
 
 
@@ -423,10 +426,7 @@ class VoidReturn(ReturnType):
         self.wrapper.write_code("return;")
 
     def write_conversion(self):
-        self.wrapper.write_code(
-            code=None,
-            failure_expression="py_retval != Py_None",
-            failure_cleanup='PyErr_SetString(PyExc_TypeError, "retval should be None");')
+        self.wrapper.add_pyret_parse_item("", "", prepend=True)
 
 argtypes.matcher.register_reverse_ret('void', VoidReturn)
 argtypes.matcher.register_reverse_ret('none', VoidReturn)
@@ -506,9 +506,12 @@ del argtype
 class IntPtrParam(Parameter):
     def __init__(self, wrapper, name, **props):
         if "direction" not in props:
-            raise ValueError("cannot use int* parameter without direction")
+            raise argtypes.ArgTypeConfigurationError(
+                "cannot use int* parameter without direction")
         if props["direction"] not in ("out", "inout"):
-            raise ValueError("cannot use int* parameter with direction '%s'" % (props["direction"],))
+            raise argtypes.ArgTypeConfigurationError(
+                "cannot use int* parameter with direction '%s'"
+                % (props["direction"],))
         Parameter.__init__(self, wrapper, name, **props)
     def get_c_type(self):
         return self.props.get('c_type', 'int*')
@@ -529,8 +532,9 @@ class GEnumReturn(IntReturn):
     def write_conversion(self):
         self.wrapper.write_code(
             code=None,
-            failure_expression=("pyg_enum_get_value(%s, py_retval, (gint *)&retval)" %
-                                self.props['typecode']))
+            failure_expression=(
+            "pyg_enum_get_value(%s, py_retval, (gint *)&retval)"
+            % (self.props['typecode'],)))
 
 argtypes.matcher.register_reverse_ret("GEnum", GEnumReturn)
 
@@ -538,9 +542,9 @@ class GEnumParam(IntParam):
     def convert_c2py(self):
         self.wrapper.add_declaration("PyObject *py_%s;" % self.name)
         self.wrapper.write_code(code=("py_%s = pyg_enum_from_gtype(%s, %s);" %
-                                      (self.name, self.props['typecode'], self.name)),
-                                cleanup=("Py_DECREF(py_%s);" % self.name),
-                                failure_expression=("!py_%s" % self.name))
+                                (self.name, self.props['typecode'], self.name)),
+                        cleanup=("Py_DECREF(py_%s);" % self.name),
+                        failure_expression=("!py_%s" % self.name))
         self.wrapper.add_pyargv_item("py_%s" % self.name)
 
 argtypes.matcher.register_reverse("GEnum", GEnumParam)
@@ -549,16 +553,18 @@ class GFlagsReturn(IntReturn):
     def write_conversion(self):
         self.wrapper.write_code(
             code=None,
-            failure_expression=("pyg_flags_get_value(%s, py_retval, (gint *)&retval)" %
-                                self.props['typecode']))
+            failure_expression=(
+            "pyg_flags_get_value(%s, py_retval, (gint *)&retval)" %
+            self.props['typecode']))
 
 argtypes.matcher.register_reverse_ret("GFlags", GFlagsReturn)
 
 class GFlagsParam(IntParam):
     def convert_c2py(self):
         self.wrapper.add_declaration("PyObject *py_%s;" % self.name)
-        self.wrapper.write_code(code=("py_%s = pyg_flags_from_gtype(%s, %s);" %
-                                      (self.name, self.props['typecode'], self.name)),
+        self.wrapper.write_code(code=(
+            "py_%s = pyg_flags_from_gtype(%s, %s);" %
+            (self.name, self.props['typecode'], self.name)),
                                 cleanup=("Py_DECREF(py_%s);" % self.name),
                                 failure_expression=("!py_%s" % self.name))
         self.wrapper.add_pyargv_item("py_%s" % self.name)
@@ -569,8 +575,9 @@ argtypes.matcher.register_reverse("GFlags", GFlagsParam)
 class GtkTreePathParam(IntParam):
     def convert_c2py(self):
         self.wrapper.add_declaration("PyObject *py_%s;" % self.name)
-        self.wrapper.write_code(code=("py_%s = pygtk_tree_path_to_pyobject(%s);" %
-                                      (self.name, self.name)),
+        self.wrapper.write_code(code=(
+            "py_%s = pygtk_tree_path_to_pyobject(%s);" %
+            (self.name, self.name)),
                                 cleanup=("Py_DECREF(py_%s);" % self.name),
                                 failure_expression=("!py_%s" % self.name))
         self.wrapper.add_pyargv_item("py_%s" % self.name)
@@ -578,6 +585,23 @@ class GtkTreePathParam(IntParam):
 argtypes.matcher.register_reverse("GtkTreePath*", GtkTreePathParam)
 
 
+class GtkTreePathReturn(ReturnType):
+    def get_c_type(self):
+        return self.props.get('c_type', 'GtkTreePath *')
+    def write_decl(self):
+        self.wrapper.add_declaration("GtkTreePath * retval;")
+    def write_error_return(self):
+        self.wrapper.write_code("return NULL;")
+    def write_conversion(self):
+        self.wrapper.write_code(
+            "retval = pygtk_tree_path_from_pyobject(py_retval);\n",
+            failure_expression=('!retval'),
+            failure_exception=(
+    'PyErr_SetString(PyExc_TypeError, "retval should be a GtkTreePath");'))
+
+argtypes.matcher.register_reverse_ret("GtkTreePath*", GtkTreePathReturn)
+
+
 class BooleanReturn(ReturnType):
     def get_c_type(self):
         return "gboolean"
@@ -588,8 +612,9 @@ class BooleanReturn(ReturnType):
         self.wrapper.write_code("return FALSE;")
     def write_conversion(self):
         self.wrapper.add_pyret_parse_item("O", "&py_main_retval", prepend=True)
-        self.wrapper.write_code("retval = PyObject_IsTrue(py_main_retval)? TRUE : FALSE;",
-                                code_sink=self.wrapper.post_return_code)
+        self.wrapper.write_code(
+            "retval = PyObject_IsTrue(py_main_retval)? TRUE : FALSE;",
+            code_sink=self.wrapper.post_return_code)
 argtypes.matcher.register_reverse_ret("gboolean", BooleanReturn)
 
 class BooleanParam(Parameter):
@@ -617,9 +642,12 @@ class DoubleParam(Parameter):
 class DoublePtrParam(Parameter):
     def __init__(self, wrapper, name, **props):
         if "direction" not in props:
-            raise ValueError("cannot use double* parameter without direction")
+            raise argtypes.ArgTypeConfigurationError(
+                "cannot use double* parameter without direction")
         if props["direction"] not in ("out", ): # inout not yet implemented
-            raise ValueError("cannot use double* parameter with direction '%s'" % (props["direction"],))
+            raise argtypes.ArgTypeConfigurationError(
+                "cannot use double* parameter with direction '%s'"
+                % (props["direction"],))
         Parameter.__init__(self, wrapper, name, **props)
     def get_c_type(self):
         return self.props.get('c_type', 'double*')
@@ -637,11 +665,7 @@ class DoubleReturn(ReturnType):
     def write_error_return(self):
         self.wrapper.write_code("return -G_MAXFLOAT;")
     def write_conversion(self):
-        self.wrapper.write_code(
-            code=None,
-            failure_expression="!PyFloat_AsDouble(py_retval)",
-            failure_cleanup='PyErr_SetString(PyExc_TypeError, "retval should be a float");')
-        self.wrapper.write_code("retval = PyFloat_AsDouble(py_retval);")
+        self.wrapper.add_pyret_parse_item("d", "&retval", prepend=True)
 
 for argtype in ('float', 'double', 'gfloat', 'gdouble'):
     argtypes.matcher.register_reverse(argtype, DoubleParam)
@@ -670,6 +694,7 @@ class GBoxedParam(Parameter):
 
 argtypes.matcher.register_reverse("GBoxed", GBoxedParam)
 
+
 class GBoxedReturn(ReturnType):
     def get_c_type(self):
         return self.props.get('c_type')
@@ -678,17 +703,64 @@ class GBoxedReturn(ReturnType):
     def write_error_return(self):
         self.wrapper.write_code("return retval;")
     def write_conversion(self):
-        self.wrapper.write_code(
+        self.wrapper.write_code(code = None,
             failure_expression=("!pyg_boxed_check(py_retval, %s)" %
                                 (self.props['typecode'],)),
-            failure_cleanup=('PyErr_SetString(PyExc_TypeError, "retval should be a %s");'
-                             % (self.props['typename'],)))
+            failure_exception=(
+            'PyErr_SetString(PyExc_TypeError, "retval should be a %s");'
+            % (self.props['typename'],)))
         self.wrapper.write_code('retval = pyg_boxed_get(py_retval, %s);' %
                                 self.props['typename'])
 
 argtypes.matcher.register_reverse_ret("GBoxed", GBoxedReturn)
 
 
+class GdkRegionPtrReturn(GBoxedReturn):
+    def write_error_return(self):
+        self.wrapper.write_code("return gdk_region_new();")
+    def write_conversion(self):
+        self.props['typecode'] = 'PYGDK_TYPE_REGION'
+        self.props['typename'] = 'GdkRegion'
+        super(GdkRegionPtrReturn, self).write_conversion()
+
+argtypes.matcher.register_reverse_ret("GdkRegion*", GdkRegionPtrReturn)
+
+
+class PangoFontDescriptionReturn(GBoxedReturn):
+    def write_error_return(self):
+        self.wrapper.write_code("return pango_font_description_new();")
+    def write_conversion(self):
+        self.props['typecode'] = 'PANGO_TYPE_FONT_DESCRIPTION'
+        self.props['typename'] = 'PangoFontDescription'
+        super(PangoFontDescriptionReturn, self).write_conversion()
+
+argtypes.matcher.register_reverse_ret("PangoFontDescription*",
+                                      PangoFontDescriptionReturn)
+
+
+class PangoFontMetricsReturn(GBoxedReturn):
+    def write_error_return(self):
+        self.wrapper.write_code("return pango_font_metrics_new();")
+    def write_conversion(self):
+        self.props['typecode'] = 'PANGO_TYPE_FONT_METRICS'
+        self.props['typename'] = 'PangoFontMetrics'
+        super(PangoFontMetricsReturn, self).write_conversion()
+
+argtypes.matcher.register_reverse_ret("PangoFontMetrics*",
+                                      PangoFontMetricsReturn)
+
+
+class PangoLanguageReturn(GBoxedReturn):
+    def write_error_return(self):
+        self.wrapper.write_code("return pango_language_from_string(\"\");")
+    def write_conversion(self):
+        self.props['typecode'] = 'PANGO_TYPE_LANGUAGE'
+        self.props['typename'] = 'PangoLanguage'
+        super(PangoLanguageReturn, self).write_conversion()
+
+argtypes.matcher.register_reverse_ret("PangoLanguage*", PangoLanguageReturn)
+
+
 class GdkRectanglePtrParam(Parameter):
     def get_c_type(self):
         return self.props.get('c_type').replace('const-', 'const ')
@@ -720,6 +792,7 @@ class PyGObjectMethodParam(Parameter):
                                 failure_expression=("!py_%s" % self.name))
         self.wrapper.set_call_target("py_%s" % self.name, self.method_name)
 
+
 class CallbackInUserDataParam(Parameter):
     def __init__(self, wrapper, name, free_it, **props):
         Parameter.__init__(self, wrapper, name, **props)
